#!/usr/bin/python

import os
import re
import sys
import md5

from libutils import *
from config import *

INTERESTING_CALLS = ["CreateMutex", "CopyFile", "CreateFile.*WRITE", "NtasdfCreateFile", "call shell32", "advapi32.RegOpenKey",
	"KERNEL32.CreateProcess", "shdocvw", "gethostbyname", "ws2_32.bind", "ws2_32.listen", "ws2_32.htons", 
	"advapi32.RegCreate", "advapi32.RegSet", "http://",
	"^([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])\.([01]?\d\d?|2[0-4]\d|25[0-5])"]

JUNK_CALLS = ['trace:file:CreateFileW L"C:\\\\\\\\windows\\\\\\\\win.ini" GENERIC_READ FILE_SHARE_READ FILE_SHARE_WRITE  creation 3 attributes 0x80',
	'L"Software\\\\\\\\Microsoft\\\\\\\\Windows\\\\\\\\CurrentVersion\\\\\\\\ThemeManager', 'RegSetValueEx.*L"Cache"', 'RegSetValueEx.*L"History"',
	'RegSetValueEx.*L"Cookies"', 'L"Software\\\\\\\\Microsoft\\\\\\\\Windows\\\\\\\\CurrentVersion\\\\\\\\Explorer\\\\\\\\User Shell Folders"',
	'L"Software\\\\\\\\Microsoft\\\\\\\\Windows\\\\\\\\CurrentVersion\\\\\\\\Explorer\\\\\\\\Shell Folders"',
	'L"System\\\\\\\\CurrentControlSet\\\\\\\\Control\\\\\\\\NetworkProvider\\\\\\\\Order"'
	'L"Software\\\\\\\\Microsoft\\\\\\\\Windows NT\\\\\\\\CurrentVersion\\\\\\\\ProfileList"'
	'"Software\\\\\\\\Wine', 'L"System\\\\\\\\CurrentControlSet\\\\\\\\Control\\\\\\\\Keyboard Layouts',
	'L"winedbg.exe"', '"RPCSSMasterMutex0x', 'c:!windows!', 
	'L"System\\\\\\\\CurrentControlSet\\\\\\\\Control\\\\\\\\NetworkProvider\\\\\\\\Order',
	'L"Software\\\\\\\\Microsoft\\\\\\\\Windows NT\\\\\\\\CurrentVersion\\\\\\\\ProfileList'
	]

def execute_command(command):
	p = os.popen(command)
	return p.readlines()

def timeout_command(command, timeout):
	import subprocess, datetime, os, time, signal
	cmd = command.split(" ")
	start = datetime.datetime.now()
	process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
	while process.poll() is None:
		time.sleep(0.2)
		now = datetime.datetime.now()
		if (now - start).seconds> timeout:
			showWarning("Killing process %d" % process.pid)
			os.kill(process.pid, signal.SIGKILL)
			os.waitpid(-1, os.WNOHANG)
			return None
		return process.stdout.readlines()
		return process.stdout.read()

def executeMalware(malware, timeout):
	os.environ["DISPLAY"] = CONFIG_X11_DISPLAY
	return timeout_command("%s %s %d" % (LAUNCHER_PATH, malware, timeout), timeout)

def analyzeMalware(malwareFile, timeout):
	data = malwareFile.file.read()
	folderMd5 = md5.md5(data).hexdigest()
	folderName = MALWARE_FOLDER + os.sep + folderMd5

	if isCgiMode():
		print "<br/>MD5 Sum: %s<br/>" % folderMd5
	
	if os.path.exists(folderName) == 1:
		showWarning("Folder already exists! File was previously analyzed?")
	else:
		os.mkdir(folderName)
	cleanFilename = cleanFile(malwareFile.filename)
	fileName = folderName + os.sep + cleanFilename

	try:
		f = file(fileName, "wb")
		f.write(data)
		f.close()
	except:
		dieError("Error saving file (%s)" % str(sys.exc_info()[1]))

	if isCgiMode():
		print "<br/>"
		print "File saved as: <b>%s</b>" % (folderMd5 + os.sep + cleanFilename)
		print "<br/>"

	try:
		return executeMalware(fileName, timeout), folderMd5, fileName
	except:
		showWarning("Error running malware!")
		if isCgiMode():
			print "<br/>"
		dieError(str(sys.exc_info()[1]))

def getStrings(filename):
	return execute_command("strings -n 4 %s" % filename)

def getSignatureForPe(pe):
	try:
		import peutils
		signatures = peutils.SignatureDatabase(PE_SIGNATURE_PATH)
		return signatures.match_all(pe)
	except:
		return ["Error getting signature: %s" % str(sys.exc_info()[1])]

def getHeaders(filename):
	try:
		import pefile
		pe = pefile.PE(filename)
		
		signatureInfo = getSignatureForPe(pe)
		peInfo = pe.dump_info()
		
		msg = ""
		if signatureInfo:
			msg = "----------Signature----------\n\n"
			try:
				for match in signatureInfo:
					msg += "%s\n" % str(match[0])
			except:
				msg += "Error getting signature: %s\n" % repr(signatureInfo)

		msg += "\n\n"
		msg += "".join(peInfo)

		return msg
	except:
		return "Error getting headers: %s" % str(sys.exc_info()[1])

def junkCall(line):
	for junk in JUNK_CALLS:
		if re.search(junk, line):
			return True
	return False

def analyzeCalls(lines):
	prevLines = []
	for line in lines:
		for mcall in INTERESTING_CALLS:
			if re.search(mcall, line):
				if line not in prevLines and not junkCall(line):
					print line
					prevLines.append(line)
